//
//  CorkBoardController.m
//  CorkBoard
//
//  Created by 荻原 剛志 on 2011.6.21.
//  Copyright 2011. All rights reserved.
//

#import "CorkBoardController.h"
#import "PinupView.h"
#import "CorkBoardView.h"

#define BoardSizeKey    @"BoardSize"
#define BoardColorKey   @"BoardColor"
#define ViewsKey        @"SubViews"
#define URLKey          @"URL"
#define RectKey         @"Rect"
#define OpaqueKey       @"Opaque"
#define PinupExtension  @"pinup"

@implementation CorkBoardController

@synthesize menu; // PinupViewで共有されるメニュー

- (NSDictionary *)dictionaryOfContents // Local Method
{
    NSArray *subviews = [mainView subviews]; // サブビューの配列
    NSMutableDictionary *root = [NSMutableDictionary dictionary];
    // 空の辞書オブジェクト。ここにプロパティリストを作る。
    NSSize sz = [mainView frame].size;
    [root setObject:NSStringFromSize(sz) forKey:BoardSizeKey];
    // CorkBoardViewのサイズを文字列に変換し、辞書に格納
    NSData *data = [NSArchiver archivedDataWithRootObject:mainView.baseColor];
    [root setObject:data forKey:BoardColorKey];
    // 背景色をアーカイブによってバイト列に変換し、辞書に登録。

    NSMutableArray *a = [NSMutableArray array]; // 空の配列
    for (PinupView *pv in subviews) { // 各サブビューについて繰り返す
        NSMutableDictionary *dic = [NSMutableDictionary dictionary];
        // 辞書dicに、画像のパス名、表示サイズ、透明度を格納する
        [dic setObject:[pv.url absoluteString] forKey:URLKey];
        [dic setObject:NSStringFromRect([pv frame]) forKey:RectKey];
        [dic setObject:[NSNumber numberWithFloat:pv.opaque]
                forKey:OpaqueKey];
        [a addObject:dic]; // 辞書dicを配列aに追加
    }
    [root setObject:a forKey:ViewsKey]; // 配列aを辞書rootに格納する
    return root;
}

- (void)readFromDictionary:(NSDictionary *)root // Local Method
{
    NSSize sz = NSSizeFromString([root objectForKey:BoardSizeKey]);
    NSRect rect = [mainView frame]; // CorkBoardViewの大きさを取り出す
    if (!NSEqualSizes(sz, rect.size)) { // 現在の大きさと異なる場合は設定
        NSWindow *win = [mainView window];
        NSRect wrect = [win frame];
        rect.size = sz;
        wrect.size = [win frameRectForContentRect:rect].size;
        [win setFrame:wrect display:YES]; // ウィンドウのサイズも変更
    }
    NSData *data = [root objectForKey:BoardColorKey];
    if (data != nil) // 背景色をバイト列から取り出す
        mainView.baseColor = [NSUnarchiver unarchiveObjectWithData:data];

    NSArray *subviews = [root objectForKey:ViewsKey]; // サブビューの配列
    for (NSDictionary *dic in subviews) {
        NSURL *url = [NSURL URLWithString:[dic objectForKey:URLKey]];
        PinupView *pv = [[PinupView alloc] initWithURL:url scale:1.0];
        if (pv) { // 画像が読めれば
            NSRect r = NSRectFromString([dic objectForKey:RectKey]);
            [pv setFrame:r];          // 大きさを設定
            [mainView addSubview:pv]; // サブビューとして追加
            [pv setMenu: menu];       // コンテキストメニューを共有
            [pv setResizable:YES];    // リサイズ可能、透明度を設定
            pv.opaque = [[dic objectForKey:OpaqueKey] floatValue];
            [pv setNeedsDisplay:YES];
        }
    }
}

- (BOOL)openURL:(NSURL *)url { // 指定したファイルを読み込む
    NSDictionary *dic = [NSDictionary dictionaryWithContentsOfURL:url];
    if (dic == nil)
        return NO; // 失敗
    [self clear:self]; // 現在の内容を消去
    [self readFromDictionary:dic];
    [[NSDocumentController sharedDocumentController]
        noteNewRecentDocumentURL:url]; // 最近読んだ書類として登録
    return YES;
}

- (IBAction)clear:(id)sender { // サブビューを消去
    NSArray *subviews = [NSArray arrayWithArray:[mainView subviews]];
    // 現在表示されているサブビューの配列を複製する
    for (NSView *v in subviews)
        [v removeFromSuperview];
    [mainView setNeedsDisplay:YES];
}

- (IBAction)save:(id)sender {
    NSSavePanel *panel = [NSSavePanel savePanel];
    [panel setAllowedFileTypes:[NSArray arrayWithObject:PinupExtension]];
    [panel setCanSelectHiddenExtension:YES]; // 拡張子を指定し、表示を選択可能に
    [panel beginSheetModalForWindow:[mainView window]
           completionHandler:^(NSInteger result) { // ブロックオブジェクト
                  if (result == NSFileHandlingPanelOKButton) {
                      NSDictionary *dic = [self dictionaryOfContents];
                      NSURL *url = [panel URL];
                      [dic writeToURL:url atomically:YES];
                      [[NSDocumentController sharedDocumentController]
                          noteNewRecentDocumentURL:url]; // 最近扱った書類
                  } // 「保存」ボタンが押されたら、プロパティリストを作成し、
              }];   // 辞書オブジェクトをシートで指定されたファイル名で書き出す
}

- (IBAction)load:(id)sender {
    NSOpenPanel *panel = [NSOpenPanel openPanel];
    [panel setAllowedFileTypes:[NSArray arrayWithObject:PinupExtension]];
    [panel setCanSelectHiddenExtension:YES]; // 拡張子を指定し、表示を選択可能に
    [panel beginSheetModalForWindow:[mainView window]
                  completionHandler:^(NSInteger result) { // ブロックオブジェクト
                      if (result == NSFileHandlingPanelOKButton)
                          (void)[self openURL:[panel URL]];
                      // 「開く」ボタンが押されたらファイルを読み込む
                  }];
}

- (IBAction)export:(id)sender {
    NSSavePanel *panel = [NSSavePanel savePanel];
    [panel setAllowedFileTypes:[NSArray arrayWithObject:@"jpg"]];
    [panel setCanSelectHiddenExtension:YES]; // 拡張子を指定し、表示を選択可能に
    [panel beginSheetModalForWindow:[mainView window]
                  completionHandler:^(NSInteger result) { // ブロックオブジェクト
                      if (result == NSFileHandlingPanelOKButton) {
                          NSData *data = [mainView screenSizeJPEGStream];
                          [data writeToURL:[panel URL] atomically:YES];
                      } // 「保存」ボタンが押されたら、JPEG形式のデータを作成し、
                  }];   // シートで指定されたファイル名で書き出す
}

@end
