//
//  CorkBoardView.m
//  CorkBoard
//
//  Created by 荻原 剛志 on 11/06/18.
//  Copyright 2011. All rights reserved.
//

#import "CorkBoardView.h"
#import "PinupView.h"
#import "CorkBoardController.h"

@implementation CorkBoardView

- (id)initWithFrame:(NSRect)frame { // 既定のイニシャライザ
    if ((self = [super initWithFrame:frame]) != nil) {
        id drag = [NSArray arrayWithObject:(NSString *)kUTTypeFileURL];
        [self registerForDraggedTypes:drag]; // ドラッグを受け入れる設定
        baseColor = [NSColor blackColor];  // 既定値で、背景は黒
    }
    return self;
}

- (BOOL)acceptsFirstResponder { return YES; } // イベントを受け取る設定

- (void)drawRect:(NSRect)dirtyRect { // ビューの描画
    [baseColor set];
    NSRectFill(dirtyRect); // 背景色ですべてを塗りつぶすだけ
}

@dynamic baseColor;
- (NSColor *)baseColor { return baseColor; }
- (void)setBaseColor:(NSColor *)color {
    baseColor = color;
    [self setNeedsDisplay:YES]; // 背景色が変更されたらビューを書き直す
}

- (void)changeColor:(id)sender { // 色の変更に対するアクションメソッド
    self.baseColor = [sender color];
}

/* Local Method */
- (NSBitmapImageRep *)screenSizeBitmapRep { // ビットマップを得る
    NSGraphicsContext *context;
    NSRect rect;
    NSSize sz = [[NSScreen mainScreen] frame].size;
    CGFloat prop = sz.width / [self frame].size.width;  // ビューとスクリーンの比
    NSBitmapImageRep *rep = [[NSBitmapImageRep alloc]
        initWithBitmapDataPlanes:NULL pixelsWide:(NSInteger)sz.width
        pixelsHigh:(NSInteger)sz.height bitsPerSample:8 samplesPerPixel:3
        hasAlpha:NO isPlanar:NO colorSpaceName:NSCalibratedRGBColorSpace
        bitmapFormat:0 bytesPerRow:0 bitsPerPixel:32];
    context = [NSGraphicsContext graphicsContextWithBitmapImageRep:rep];
    // 空のビットマップ領域を作成してカレントコンテキストとして設定
    [NSGraphicsContext saveGraphicsState];
    [NSGraphicsContext setCurrentContext:context];
    [context saveGraphicsState];
    rect.origin = NSZeroPoint;
    rect.size = sz;
    [baseColor set];
    NSRectFill(rect); // 背景色ですべてを塗りつぶす
    for (PinupView *pv in [self subviews]) {
        [pv drawScreenSize:prop]; // サブビューを順番に描く
    }
    [context restoreGraphicsState];
    [NSGraphicsContext restoreGraphicsState];
    return rep;
}

- (NSData *)screenSizeJPEGStream { // JPEG形式のデータを取り出す
    NSBitmapImageRep *rep = [self screenSizeBitmapRep]; // ビットマップ
    NSDictionary *dic = [NSDictionary  // JPEGの圧縮オプション
                         dictionaryWithObject:[NSNumber numberWithFloat:0.85]
                         forKey:NSImageCompressionFactor];
    return [rep representationUsingType:NSJPEGFileType properties:dic];
}

- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
{ // 何かをドラッグしたマウスカーソルがビューの上に来た時に呼び出される
    NSPasteboard *pb = [sender draggingPasteboard]; // ペーストボード
    NSArray *imageTypes = [NSImage imageFileTypes]; // 画像の拡張子たち
    URLList = [NSMutableArray array]; // 空の配列
    for (NSPasteboardItem *item in [pb pasteboardItems]) {
        NSString *str = [item stringForType:(NSString *)kUTTypeFileURL];
        if ([imageTypes containsObject:[str pathExtension]])
            [URLList addObject: [NSURL URLWithString:str]];
    } // ペーストボード内の要素を取り出す繰り返し処理。要素がファイル名で、
    // 画像を表す拡張子だったらNSURLオブジェクトにして配列に格納
    return ([URLList count] > 0) // ドラッグを受け入れるかどうかを返す
            ? NSDragOperationCopy : NSDragOperationNone;
}

- (void)draggingExited:(id <NSDraggingInfo>)sender {
    URLList = nil; // マウスカーソルがビューの外に出てしまった
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{ // ドラッグ＆ドロップの操作が終わった時に呼び出される
    if (URLList == nil)
        return NO; // 有効な情報がなかった。失敗。
    int count = 0;
    CGFloat w = [[NSScreen mainScreen] frame].size.width;
    CGFloat prop = [self frame].size.width / w;  // スクリーンとビューの比
    NSPoint p = [sender draggingLocation];  // ドロップされた座標
    p = [self convertPoint:p fromView:nil]; // ビュー上の座標に変換する
    for (NSURL *url in URLList) { // 配列から要素を取り出して繰り返す
        PinupView *view = [[PinupView alloc] initWithURL:url scale:prop];
        if (view) { // 得られたURLを指定して PinupViewを作成
            [view setFrameOrigin:p]; // 表示位置はドロップされた所
            [self addSubview:view];  // サブビューとして追加する
            [view setMenu: controller.menu]; // コンテキストメニューを共有
            [view setResizable:YES]; // リサイズの方法を指定
            [view setNeedsDisplay:YES]; // 表示すべきであるという指定
            count++;
        }
    }
    return (count > 0); // 一つ以上のファイルが読み込めたら成功
}

@end
