//
//  PinupView.m
//  CorkBoard
//
//  Created by 荻原 剛志 on 11/06/18.
//  Copyright 2011. All rights reserved.
//

#import "PinupView.h"
#import "CorkBoardController.h"

@implementation PinupView

@synthesize url, opaque;

- (id)initWithURL:(NSURL *)arg scale:(CGFloat)prop {
    NSImage *img = [[NSImage alloc] initWithContentsOfURL:arg];
    if (img == nil)
        return nil; // エラー。画像が読み込めなかった
    NSRect rect = NSZeroRect;
    origSize = [img size];
    rect.size.width = origSize.width * prop; // propは縮尺
    rect.size.height = origSize.height * prop;
    if ((self = [self initWithFrame:rect]) != nil) {
        image = img;
        [image setScalesWhenResized:YES];
        [image setSize:rect.size]; // propで指定された大きさに縮小
        url = arg;
        opaque = 1.0; // 不透明100％
    }
    return self;
}

- (void)drawRect:(NSRect)dirtyRect {
    NSRect r = [self frame];
    r.origin = NSZeroPoint;
    [image setSize:r.size]; // 画像を改めてビューの大きさに縮小
    [image drawAtPoint:NSZeroPoint fromRect:r
             operation:NSCompositeSourceOver fraction:opaque];
} // 画像を描く。透明部分は合成して、自分が前面になるようにする。

// スクリーンのサイズで描画を行うためのメソッド。ratio は倍率。
- (void)drawScreenSize:(CGFloat)ratio {
    NSRect r = [self frame];
    NSPoint pt = NSMakePoint(r.origin.x * ratio, r.origin.y * ratio);
    NSRect sc = NSMakeRect(0.0, 0.0,
        r.size.width * ratio, r.size.height * ratio);
    [image setSize:sc.size]; // 画像のサイズを変更
    [image drawAtPoint:pt fromRect:sc
             operation:NSCompositeSourceOver fraction:opaque];
}

- (BOOL)acceptsFirstResponder { return YES; } // イベントを受け取る設定

- (void)setResizable:(BOOL)flag { // 縦横、周囲の余白も均等に縮小する
    static const NSUInteger ResizableMask = 
    NSViewMinXMargin | NSViewWidthSizable| NSViewMaxXMargin 
    | NSViewMinYMargin | NSViewHeightSizable | NSViewMaxYMargin;
    [self setAutoresizingMask: flag ? ResizableMask : NSViewNotSizable];
}

- (void)mouseDown:(NSEvent *)event { // マウスがクリックされた時。
    firstPt = [event locationInWindow]; // クリックされた点の座標を得る
    firstOrig = [self frame].origin; // ビューの現在位置を得る
}

- (void)mouseDragged:(NSEvent *)event { // マウスがドラッグされた。
    NSPoint p = [event locationInWindow]; // マウスの座標を得る
    NSPoint neworig = firstOrig;
    neworig.x += p.x - firstPt.x;
    neworig.y += p.y - firstPt.y; // クリックされた位置と現在位置から
    [self setFrameOrigin:neworig]; // ビューの新しい位置を設定する
    [self setNeedsDisplay:YES];
}

- (void)magnifyWithEvent:(NSEvent *)event // ピンチ操作を検出した
{
    const NSUInteger gMask = NSEventMaskEndGesture | NSEventMaskMagnify;
    CGFloat ratio = [image size].width / origSize.width; // 画像の縮尺
    NSWindow *win = [self window];
    do { // ピンチ操作が継続する間、繰り返す
        CGFloat mag = [event magnification];
        CGFloat t = ratio * (1.0 + mag); // 画像の新しい縮尺を計算
        NSSize sz = NSMakeSize(origSize.width * t, origSize.height * t);
        if (sz.width > 5.0 && sz.height > 5.0) {
            [image setSize:sz]; // あまり小さくなりすぎていなければ
            [self setFrameSize:sz]; // 画像とビューを両方ともリサイズ
            ratio = t;
            [self setNeedsDisplay:YES];
        }
        event = [win nextEventMatchingMask: gMask];
    }while ([event type] == NSEventTypeMagnify);
}

- (void)scrollWheel:(NSEvent *)event // スクロールホイールの動きを検知
{
    const double factor = 50.0; // 適当な倍率
    opaque += [event deltaY] / factor; // 不透明率を増減させる
    if (opaque > 1.0) opaque = 1.0;
    else if (opaque < 0.15) opaque = 0.15; // 透明になりすぎないように
    [self setNeedsDisplay:YES];
}

- (IBAction)changeOrder:(id)sender
{   // いったんスーパービューから取り除いて、追加し直す
    NSView *supview = [self superview];
    NSWindowOrderingMode mode; // 最前面か、最背面か
    mode = ([sender tag] == 0) ? NSWindowAbove : NSWindowBelow;
    [self removeFromSuperview];
    [supview addSubview:self positioned:mode relativeTo:nil];
}

- (IBAction)delete:(id)sender { [self removeFromSuperview]; }

@end
